import html, { HTMLElement } from "node-html-parser";
import request from "request-promise-native";
import { db } from "../../db";
import { Response } from "request";
import { IAggregator, INewsItem } from "./aggregator";

const updatePage = "http://com3d2.jp/update/";
const changeLogPattern = /\[\s*([^\s\]]+)\s*\]\s*((・.*)\s+)+/gim;

function getVersionNumber(verStr: string) {
    let verPart = verStr.replace(/[\.\s]/g, "");
    if(verPart.length < 4)
        verPart += "0";
    return +verPart;
}

async function aggregate() {
    let lastVersion = db.get("lastCOMJPVersion").value() as number;
    
    try {
        let mainPageRes = await request(updatePage, {resolveWithFullResponse: true}) as Response;

        if(mainPageRes.statusCode != 200)
            return;
        
        let rootNode = html.parse(mainPageRes.body, {
                pre: true,
                script: false,
                style: false
        });

        if(!(rootNode instanceof HTMLElement))
            return;

        let readme = rootNode.querySelector("div.readme");

        if(!readme) {
            console.log("[COM3D2 JP UPDATE] Failed to find listing!");
        }

        let latestVersionChangelog = changeLogPattern.exec(readme.text);

        if(!latestVersionChangelog)
            return [];

        let version = getVersionNumber(latestVersionChangelog[1]);
        let text = latestVersionChangelog[0];

        if(version <= lastVersion)
            return [];
        
        db.set("lastCOMJPVersion", version).write();

        return [{
            id: `com3d2-jp-updates-${version}`,
            link: updatePage,
            title: latestVersionChangelog[1],
            author: "COM3D2 UPDATE",
            contents: text,
            embedColor: 0xcccccc
        }] as INewsItem[];
    } catch(err) {
        return [];
    }
}

export default {
    aggregate: aggregate
} as IAggregator;