import html, { HTMLElement } from "node-html-parser";
import request from "request-promise-native";
import { db } from "../../db";
import { Response } from "request";
import { INewsItem, IAggregator } from "./aggregator";

const urlPattern = /diary\.php\?no=(\d+)/i;
const kissDiaryRoot = "http://www.kisskiss.tv/kiss";

async function aggregate() {
    let lastDiary = db.get("latestKissDiaryEntry").value() as number;
    
    try {
        let mainPageRes = await request(`${kissDiaryRoot}/diary.php`, {resolveWithFullResponse: true}) as Response;
        
        if(mainPageRes.statusCode != 200)
            return [];

        let rootNode = html.parse(mainPageRes.body, {
                pre: true,
                script: false,
                style: false
        });

        if(!(rootNode instanceof HTMLElement))
            return;

        let diaryEntries = rootNode.querySelectorAll("div.blog_frame_middle ul.disc li a");

        if(!diaryEntries) {
            console.log("[KISS DIARY] Failed to find listing!");
        }

        let result : INewsItem[] = [];
        let latestEntry = lastDiary;

        for(let a of diaryEntries) {
            let matches = urlPattern.exec(a.rawAttributes.href);
            if(!matches)
                continue;
            
            let id = +matches[1];

            if(id <= lastDiary)
                continue;

            if(id > latestEntry)
                latestEntry = id;

            let diaryLink = `${kissDiaryRoot}/${a.rawAttributes.href}`;
            let res = await request(diaryLink, {resolveWithFullResponse: true}) as Response;
            if(res.statusCode != 200)
                continue;

            let node = html.parse(res.body, {
                pre: true,
                script: false,
                style: false
            });

            if(!(node instanceof HTMLElement))
                continue;

            let title = node.querySelector("table.blog_frame_top tr td a");
            let contents = node.querySelector("div.blog_frame_middle");
            let bottomFrame = contents.querySelector("div.blog_data");
            if(bottomFrame) {
                let child = contents.childNodes[0];
                if(child instanceof HTMLElement)
                    child.removeChild(bottomFrame);
            }

            result.push({
                id: `kisskisstv-diary-${id}`,
                link: diaryLink,
                title: title.text,
                author: "KISS BLOG",
                contents: contents.innerHTML,
                embedColor: 0xf4c100
            });
        }

        db.set("latestKissDiaryEntry", latestEntry).write();
        return result;
    } catch(err) {
        return [];
    }
}

export default {
    aggregate: aggregate
} as IAggregator;