import { db } from "./db"
import { CollectionChain } from "lodash";
import { GuildMember } from "discord.js";
import { DocumentationSet } from "./commands/command";
import { getRepository } from "typeorm";
import { KnownUser } from "./entity/KnownUser";

const VALID_EXTENSIONS = new Set([
    "png",
    "jpg",
    "jpeg",
    "bmp",
]);

export let documentation : DocumentationSet = {};

export function isDevelopment() {
    return process.env.NODE_ENV == "dev";
}

export function shouldShowMaintenanceMessage(serverId : string) {
    if(process.env.NODE_ENV != "dev")
        return false;

    let devServers = db.get("devServers") as CollectionChain<string>;
    return !devServers.includes(serverId).value();
}

export function isValidImage(fileName: string) {
    let extPosition = fileName.lastIndexOf(".");
    if(extPosition < 0)
        return false;
    let ext = fileName.substring(extPosition + 1).toLowerCase();
    return VALID_EXTENSIONS.has(ext);
}

export function isAuthorised(member : GuildMember) {
    let users = db.get("editors.users") as CollectionChain<string>;
    let roles = db.get("editors.roles") as CollectionChain<string>;
    if (users.includes(member.id).value())
        return true;
    if (roles.intersectionWith(member.roles.keyArray()).isEmpty().value())
        return false;
    return true;
}

export async function isAuthorisedAsync(member : GuildMember) {
    let repo = getRepository(KnownUser);

    let user = await repo.findOne({
        where: { userID: member.id },
        select: [ "canModerate" ]
    });

    if (user && user.canModerate)
        return true;

    let role = await repo.createQueryBuilder()
                .select(["userId"])
                .where("userId in (:...ids)", {ids: member.roles.keyArray()})
                .andWhere("canModerate = 1")
                .getOne();
    if (role)
        return true;
    return false;
}